import tkinter
import math
import cmath

class Farben:
    """
    erste primitive Version mit nur wenigen Farben 
    Die Farben können über die deutschen Namen oder über
    Zahlen abgerufen werden
    Hinweis:   "#fff" entspricht "weiss", 
         statt "#xxx" kann auch "red", "green"  usw. benutzt werden
    andere Lösungen mit true-colors sind denkbar
    """
    def __init__(self):
        """
        transparent, schwarz, blau, gruen, tuerkis, rot, gelb, grau, weiss,
        ergänzt um braun und orange
        """
        self.Fnamen = ["transparent", \
                       "schwarz" , \
                       "weiss"    , \
                       "blau"   , \
                       "braun" , \
                       "cyan"     , \
                       "gelb"    , \
                       "grau"    , \
                       "gruen"   ,\
                       "magenta"   ,\
                       "maroon",  \
                       "orange",  \
                       "rot",  \
                       "pink",  \
                       "sienna",  \
                       "tuerkis"  \
                      ]

        self.Farbe={ \
                      "transparent":"", \
                      "schwarz"    :"#000", \
                      "weiss"      :"#fff", \
                      "blau"       :"#00f", \
                      "braun"      :"#a52a2a", \
                      "cyan"       :"#0ff", \
                      "gelb"       :"#ff0", \
                      "grau"       :"#ccc", \
                      "gruen"      :"#0f0", \
                      "magenta"    :"#f0f", \
                      "maroon"     :"#8e236b", \
                      "orange"     :"#f80", \
                      "rot"        :"#f00", \
                      "pink"       :"#ffc0cb", \
                      "sienna"     :"#8e6b23", \
                      "tuerkis"    :"#0ee", \
                   }

    def gibFarbeNr(self,nr):
        """
        nr : int : 0 ..  für die oben angegebenen Farben
        liefert die Farbdarstellung für X
        """
        try:
            farbe = self.Farbe[self.gibFarbnamen(nr)]
        except KeyError as e:
            farbe = "#333"
        return farbe

    def gibFarbnamen(self,nr):
        """
        nr : int : 0 ..   für die oben angegebenen Farben
        liefert den (deutschen) Bezeichner der Farbnummer (s.o.)
        """
        try:
            farbname = self.Fnamen[nr]
        except KeyError as e:
            farbname = "dunkelgrau"
        return farbname

    def gibFarbe(self,wort):
        """
        wort : string :  ein Element aus den oben angegebenen Farben
        liefert die Farbdarstellung fuer X
        """
        try:
            farbe = self.Farbe[wort]
        except KeyError as e:
            farbe = "#333"
        return farbe

    def zeigeFarbnamen():
        '''Methode der Klasse'''
        farbnamen=""
        for name in Farben().Fnamen:
            farbnamen+=name+"\n"
        return farbnamen

class _Leinwand:
    """
    Eine Leinwand, auf die gezeichnet werden kann. Die einzelnen Formen
    benutzen diese Leinwand, um sich darauf selbst zu zeichnen. 
    """
    _instance=None

    def __init__(self,width='18c',height='14c',bg='weiss'):
        """
        Erstellt das Fenster mit der Leinwand für das Möbelprojekt.
        width: die Breite der Leinwand; Standard: '18c' für 18cm
        height: die Höhe der Leinwand; Standard: '14c' für 14cm
        bg: die Hintergrundfarbe; Standard: 'weiss'
        """
        if _Leinwand._instance:
            raise Error("Nur eine Leinwand erlaubt!")
        else:
            _Leinwand._instance=self
            self.fenster=tkinter.Tk()
            self.farben=Farben()
            self.fenster.title('Möbelprojekt Grafik')
            self.canvas=tkinter.Canvas(self.fenster,width=width,height=height,\
                                       bg=self.farben.gibFarbe(bg))
            self.canvas.pack()

    def zeichne(self,form):
        """
        Zeichnet die Form auf die Leinwand.
        form: die zu zeichnenende Form
        """
        return form.zeichne(self)

    def loesche(self,item):
        """
        Entfernt die Form von der Leinwand.
        form: die zu entfernende Form
        """
        return self.canvas.delete(item)

    ## Abschnitt fuer Formen
    def polygon(self,*coords,**kw):
        """
        Zeichnet ein Polygon auf die Leinwand.
        coords: die Koordinaten der Eckpunkte des Polygons in X-Y-Paaren
        """
        return self.canvas.create_polygon(*coords,**kw)

    def ellipse(self,x1,y1,x2,y2,**kw):
        """
        Zeichnet eine Ellipse auf die Leinwand.
        x1,y1: die x- und y-Koordinate der linken oberen Ecke des Rechtecks, 
               welches die Ellipse umfasst
        x2,y2: die x- und y-Koordinate der rechten unteren Ecke des Rechtecks, 
               welches die Ellipse umfasst
        Problem ist das Fehlen der Drehbarkeit bei tkinter
        Loesungsansatz ein Polygon mit smooth
        """
        b,h=x2-x1,y2-y1
        return self.canvas.create_polygon(x1,y1+h/2,
                                          x1+b/8, y1+self.fO(b/8,b,h),
                                          x1+b/2, y1+self.fO(b/2,b,h),
                                          x1+7*b/8, y1+self.fO(7*b/8,b,h),
                                          x1+b, y1+h/2,
                                          x1+7*b/8, y1+self.fU(7*b/8,b,h),
                                          x1+b/2, y1+self.fU(b/2,b,h),
                                          x1+b/8, y1+self.fU(b/8,b,h),
                                          smooth=1,
                                          **kw)
    ## dazu Hilfsfunktionen
    def fO(self,x,b,h): return h/2-h/b*math.sqrt((b/2)**2-((b/2)-x)**2)
    def fU(self,x,b,h): return h/2+h/b*math.sqrt((b/2)**2-((b/2)-x)**2)

    def linie(self,x1,y1,x2,y2,**kw):
        """
        Zeichnet eine Linie auf die Leinwand.
        x1,y1: die x- und y-Koordinate des Startpunkts
        x2,y2: die x- und y-Koordinate des Endpunkts
        """
        return self.canvas.create_line(x1,y1,x2,y2,**kw)

### Problem: Es ist nicht wirklich ein OO Ansatz, da die Winkel absolut
### gesetzt werden. Warum andere Winkel als 0,90,180,270
### nur einen Strich liefern, habe ich nicht heraus gefunden.
    def bogen(self,x1,y1,x2,y2,start=-90,extent=180,**kw):
        """
        Zeichnet einen Halbkreisbogen auf die Leinwand.
        x1,y1: die x- und y-Koordinate des oberen Endes des Durchmessers
        x2,y2: die x- und y-Koordinate des unteren Endes des Durchmessers
        """
        r=math.sqrt((x1-x2)**2+(y1-y2)**2)/2
        xm,ym=(x1+x2)/2,(y1+y2)/2
        xs,xe=xm-r,xm+r
        ys,ye=ym+r,ym-r
        return self.canvas.create_arc(xs,ys,xe,ye,start=start,extent=extent,style=tkinter.ARC,**kw)
    
    ## veraendernde Methoden
    def skalieren(self,item,x,y,xFaktor,yFaktor):
        """
        Ändert die Größer einer Form auf der Leinwand.
        item: die zu ändernde Form
        x,y: der Mittelpunkt für die Skalierung
        xFaktor: der Faktor, um den in x-Richtung gestreckt werden soll
        yFaktor: der Faktor, um den in y-Richtung gestreckt werden soll
        """
        return self.canvas.scale(item,x,y,xFaktor,yFaktor)

    def bewegen(self,item,x,y):
        """
        Bewegt eine Form auf der Leinwand.
        item: die zu bewegende Form
        x: die Entfernung, um die in x-Richtung bewegt werden soll
        y: die Entfernung, um die in y-Richtung bewegt werden soll
        """
        return self.canvas.move(item,x,y)
    
    def farbeAendern(self,item,neueFarbe):
        """
        Ändert die Farbe einer Form auf der Leinwand.
        item: die Form, die umgefärbt werden soll. Wichtig: Eine Linie hat nur
              eine Randfarbe!
        neueFarbe: die neue Farbe für diese Form, dies muss eine gültige Farbe
                   aus der Klasse Farben sein
        """
        if self.canvas.type(item)=='line':
            return
        else:
            return self.canvas.itemconfig(item,fill=self.farben.gibFarbe(neueFarbe))

    def randfarbeAendern(self,item,neueFarbe):
        """
        Ändert die Randfarbe einer Form auf der Leinwand.
        item: die Form, die umgefärbt werden soll. Wichtig: Eine Linie hat nur
              eine Randfarbe!
        neueFarbe: die neue Farbe für den Rand dieser Form, dies muss eine
                   gültige Farbe aus der Klasse Farben sein
        """
        if self.canvas.type(item)=='line':
            return self.canvas.itemconfig(item,fil=self.farben.gibFarbe(neueFarbe))
        else:
            return self.canvas.itemconfig(item,outline=self.farben.gibFarbe(neueFarbe))

    def drehe(self,item,x,y,winkel):
        
        cwinkel=cmath.exp(winkel*math.pi/180*1j)
        coords=self.canvas.coords(item)
        newcoords=[]
        zentrum=complex(x,y)
        X=None
        for Y in coords:
            if X is None:
                X=Y
            else:
                neuer_punkt=cwinkel*(complex(X,Y)-zentrum)+zentrum
                newcoords.append(neuer_punkt.real)
                newcoords.append(neuer_punkt.imag)
                X=None
        self.canvas.coords(item,*newcoords)

def Leinwand(singleton=_Leinwand()):
    """
    Liefert die Leinwand. Hiervon darf es nur ein einziges Objekt geben,
    deswegen liefert diese Funktion bei jedem Aufruf dieselbe Leinwand
    zurück. Dies ist nötig, damit die einzelnen Objekte auf dieselbe
    Leinwand gezeichnet werden anstatt eine Leinwand pro Objekt zu haben.
    """
    return singleton
