## moebelgruppe.py
"""fuenfte Version: V4 erweitert:
Einführung der Methode _aktualisiereAbmessungen"""
from leinwand import Leinwand
from abc import ABC,abstractmethod
from moebel import Moebel

class Moebelgruppe(ABC, Moebel):
    """
    Abstrakte Oberklasse
    """
    def __init__(self,
                 xPosition=50,
                 yPosition=50,
                 winkel=0,
                 randfarbe='black'):
        """
        Erzeugt eine Moebelgruppe an der gegebenen Position.
        xPosition: die x-Koordinate der Mitte der Moebelgruppe
        yPosition: die y-Koordinate der Mitte der Moebelgruppe
        randfarbe: die Randfarbe, in der der moebel angezeigt werden soll.
               Dies sollte einer der Farbnamen der Klasse Farben sein.  
        """
        self.__moebel=[]
        super().__init__(xPosition,yPosition,0,0,winkel,randfarbe)

    def zeige(self):
        """
        Zeigt alle Möbel der Gruppe an.
        """
        for moebel in self.__moebel:
            moebel.zeige()

    def verberge(self):
        """
        Verbirgt alle Möbel der Gruppe.
        """
        for moebel in self.__moebel:
            moebel.verberge()

    def horizontalBewegen(self,entfernung):
        """
        Bewegt alle Möbel in horizontaler Richtung auf der Leinwand.
        entfernung: die Anzahl an Pixeln, um die das Möbel nach rechts
                    bewegt werden soll. Für Bewegungen nach links sind
                    negative Zahlen zu verwenden.
        """
        for moebel in self.__moebel:
            moebel.horizontalBewegen(entfernung)
        
    def vertikalBewegen(self,entfernung):
        """
        Bewegt alle Möbel in vertikaler Richtung auf der Leinwand.
        entfernung: die Anzahl an Pixeln, um die das Möbel nach unten
                    bewegt werden soll. Für Bewegungen nach oben sind
                    negative Zahlen zu verwenden.
        """
        for moebel in self.__moebel:
            moebel.vertikalBewegen(entfernung)

    def drehe(self,umWinkel):
        """
        Dreht alle Möbel.
        umWinkel: der Winkel in Grad, um den im Uhrzeigersinn gedreht werden soll
        """
        for moebel in self.__moebel:
            for form in moebel.gibFormen():
                Leinwand().drehe(form,
                                 self.gibX()+self.gibB()/2,
                                 self.gibY()+self.gibT()/2,
                                 umWinkel)

    def erzeugeFigur(self):
        """
        Erzeugt die Figur.
        """
        x,y,b,t,w,lw = self.gibX(),self.gibY(),self.gibB(),self.gibT(),self.gibW(),self.gibLW()
        xm,ym=x+b/2,y+t/2
        formen=[]
        if not self.gibFormen():
            for moebel in self.__moebel:
                formen+=moebel.erzeugeFigur()
            for form in formen:
                lw.bewegen(form,x,y)
            return formen

    def fuegeHinzu(self, moebel):
        '''fuegt ein moebel am Ende der Liste hinzu,
        raeumlich aber nicht positionsgerecht'''
        if moebel in self.__moebel: return
        self.verberge()
        self.__moebel.append(moebel)
        self._AktualisiereAbmessungen()
        self.zeige()
    
    def entferne(self, moebel):
        '''entfernt das uebergebene Schrankobjekt'''
        if not(moebel in self.__moebel): return
        self.verberge()
        self.__moebel.remove(moebel)
        self._AktualisiereAbmessungen()
        self.zeige()

    def _AktualisiereAbmessungen(self):
        """setzt breite und tiefe neu
        die hinzugefuegten moebelobjekte muessen mit Winkel der Gruppe 0
        positionsgerecht in die Moebelgruppe eingefuegt werden"""
        b,t=0,0
        for moebel in self.__moebel:
            b=max(b,moebel.gibB())
            t=max(t,moebel.gibT())
        self.breiteAendern(b)
        self.tiefeAendern(t)

    def gibMoebel(self, position):
        '''gibt das Moebelobjekt an der Position der Liste zurueck
        Position 1 bis laenge'''
        if position<=0: return None
        if position>len(self.__moebel): return None
        return self.__moebel[position-1]


    def gibAlle(self):
        return self.__moebel
